@@ -1,4 +1,4 @@
-Heading <- function(name = NULL) 
+Heading <- function(name = NULL, override = TRUE) 
     stop("This is a pseudo-function, not meant to be called.")
     
 Justify <- function(labels, data=labels)
@@ -14,3 +14,6 @@ Percent <- function(denom = "all", fn = percent)
     stop("This is a pseudo-function, not meant to be called.")
 
 percent <- function(x, y) 100*length(x)/length(y)
+
+Arguments <- function (...)
+    stop("This is a pseudo-function, not meant to be called.")
